
INCLUDE(${CMAKE_MODULE_PATH}/LIB.cmake)

ENABLE_TESTING()

SET(SIGNAL_CMD "$ENV{Signal_bin}/signal")
SET(UNDEFINED_PARAMETER  "xxxyyyzzz")
SET(MESS1 "## ERROR ##: cannot compile missing file")
SET(USER_SELECTED_OPTION)

SET(LIST_TEST_SYSTEMATIC_OPTION 
  sme sme "BASIC POLY"
  ssme ssme "BASIC POLY"
  dr dr "BASIC POLY"
#  crew=bc crewbc "BASIC POLY"
  crew=c crewc "BASIC POLY"
  crew=b crewb "BASIC POLY"
  crew crew "BASIC POLY"
  su su "BASIC POLY"
  s=1 s1 "BASIC POLY"
  s=3 s3 "BASIC POLY"
  ec ec "BASIC POLY"
  ec=2 ec2 "BASIC POLY"
  ds ds "BASIC POLY"
  pdg pdg "BASIC POLY"
  sbo sbo "BASIC POLY"
  sso sso "BASIC POLY"
  force force "BASIC POLY"
  check check "BASIC POLY"
  udo udo "BASIC POLY"
  umd umd "BASIC POLY"
  bC bC "BASIC POLY BOOL"
  endo endo "BASIC POLY ENDO"
  clu clu "BASIC POLY BOOL ENDO"
  threads threads "BASIC POLY BOOL ENDO"
  sD sD "BASIC POLY BOOL ENDO SCH"
  c c "BASIC POLY BOOL ENDO SCH"
  c++ cpp "BASIC POLY BOOL ENDO SCH"
  java java "BASIC POLY BOOL ENDO SCH"
  flat flat "BASIC POLY ENDO BOOL FLAT"
  z3z z3z "BASIC POLY F3_POLY"
  syndex syndex "BASIC POLY BOOL ENDO FLAT"
  )

  
FUNCTION(GET_OPTIONS_TO_TEST OPTIONS_TO_BE_TESTED)
  IF(DEFINED USER_SELECTED_OPTION)
    SET(${OPTIONS_TO_BE_TESTED} ${USER_SELECTED_OPTION} PARENT_SCOPE)
  ELSE()
    SET(${OPTIONS_TO_BE_TESTED} ${LIST_TEST_SYSTEMATIC_OPTION} PARENT_SCOPE)
  ENDIF()
ENDFUNCTION()

FUNCTION(GET_COMMAND_TEST_POST COMMAND_TEST_POST)
  IF(UNIX)
    SET(${COMMAND_TEST_POST} "grep" "SUCCESS" "${TARGETDIR}/Signal.log"  PARENT_SCOPE)
  ELSE(UNIX)
    SET(${COMMAND_TEST_POST}  "grep" "SUCCESS" "${TARGETDIR}/Signal.log"  PARENT_SCOPE)
  ENDIF(UNIX)
ENDFUNCTION()


FUNCTION( GET_TEST_NAME COMPLETENAME TEST_NAME)

  STRING(REPLACE "$ENV{pK_Examples}" TEST_ VTSR ${COMPLETENAME})
  
  SET(${TEST_NAME} ${VTSR} PARENT_SCOPE)
  
ENDFUNCTION()

FUNCTION(CHECKFILESFUNC TARGETDIR NAME CHECKFILES)
  #   MESSAGE("---------------------------------")
  #   MESSAGE("TARGETDIR= ${TARGETDIR}")
  #   MESSAGE("NAME =${NAME}")
  #   MESSAGE("CHECKFILES=${CHECKFILES}")
  
  FOREACH(SUFFIX ${CHECKFILES})
    STRING(STRIP ${SUFFIX} SUFFIX)
    SET(COMMAND_TEST ${SIGNAL_CMD} ${NAME}_${SUFFIX}.SIG)
    # MESSAGE("COMMAND_TEST = ${SIGNAL_CMD} ${NAME}_${SUFFIX}.SIG  ")
    GET_TEST_NAME(${TARGETDIR}/${NAME}_POST_CHECK_${SUFFIX} TEST_NAME)
    ADD_TEST(NAME ${TEST_NAME}
      WORKING_DIRECTORY "${TARGETDIR}/${NAME}"
      COMMAND ${COMMAND_TEST}
      )    
  ENDFOREACH()
  
ENDFUNCTION()

# It adds the test called ${TARGETDIR}/${NAME}
#   - NAME : prefix of the file of the tested program
#   - PARAM: filename of  the parameter or UNDEFINED_PARAMETER
#   - SRCDIR: input file directory path
#   - OPTION: is the compiler option string without initial "-", 
#   - TRGTDIRPREFIX: is a prefix of the target directory path
#   - OPTIONSUFFIX is a not empty string
#   - CHECKFILES is a (possibly empty) list of not empty strings
# Action:
#     If PARAM is defined (and exists) the following compilation command is added to the test set:
#              Signal_bin/signal SRCDIR/NAME.SIG -par=SRCDIR/NAME.PAR -OPTION -d=TRGTDIRPREFIX_OPTIONSUFFIX;
#       else the following compilation command is to the test set:
#              Signal_bin/signal SRCDIR/NAME.SIG -OPTION -d=TRGTDIRPREFIX_OPTIONSUFFIX;
#       a compilation log file TRGTDIRPREFIX_OPTIONSUFFIX/Signal.log is generated;
#       the generated files are checked by calling
#              pK_Examples/checkAllGeneratedFiles TRGTDIRPREFIX_OPTIONSUFFIX/ NAME {TRASTRING}
FUNCTION(GEN_ONE_FILE_OPTION NAME PARAM SRCDIR OPTION TRGTDIRPREFIX OPTIONSUFFIX CHECKFILES)

  SET(SIG_FILE ${SRCDIR}/${NAME}.SIG)
  IF (NOT EXISTS ${SIG_FILE})
    PK_ERROR(" GEN_ONE_FILE_OPTION (1), ${MESS1} ${SIG_FILE}")
    RETURN()
  ENDIF(NOT EXISTS ${SIG_FILE})

  IF("${PARAM}" STREQUAL "${UNDEFINED_PARAMETER}")
    SET(PARAM_FILE " ")
  ELSE("${PARAM}" STREQUAL "${UNDEFINED_PARAMETER}")
    SET(PARAM ${SRCDIR}/${PARAM})
    IF (NOT EXISTS ${PARAM})
      PK_ERROR("GEN_ONE_FILE_OPTION (2), ${MESS1} ${PARAM}")
      RETURN()
    ELSE(NOT EXISTS ${PARAM})
      SET(PARAM_FILE "-par=${PARAM}")
    ENDIF(NOT EXISTS ${PARAM})
  ENDIF("${PARAM}" STREQUAL "${UNDEFINED_PARAMETER}")
  
  SET(TARGETDIR ${TRGTDIRPREFIX}/${OPTIONSUFFIX})
  FILE(MAKE_DIRECTORY ${TARGETDIR})
  SET(COMMAND_TEST ${SIGNAL_CMD} ${NAME}.SIG ${PARAM_FILE} "-tra -${OPTION}" "-d=${TARGETDIR}")
  GET_TEST_NAME(${TARGETDIR}/${NAME} TEST_NAME)
  ADD_TEST(NAME ${TEST_NAME} 
    WORKING_DIRECTORY "${SRCDIR}" 
    COMMAND ${COMMAND_TEST}
    )
  GET_COMMAND_TEST_POST(COMMAND_TEST_POST)
  PRINT(" COMMAND_TEST_POST = ${COMMAND_TEST_POST}")
  GET_TEST_NAME( ${TARGETDIR}/${NAME}_POST TEST_NAME)
  ADD_TEST(NAME ${TEST_NAME}
    WORKING_DIRECTORY "${SRCDIR}" 
    COMMAND ${COMMAND_TEST_POST}
    )
  CHECKFILESFUNC("${TARGETDIR}" "${NAME}" "${CHECKFILES}")
  # not possible: ADD_DEPENDENCIES( ${TARGETDIR}/${NAME}_POST  ${TARGETDIR}/${NAME})
  
ENDFUNCTION()

# It adds a test called NAMETEST, for the program SRCDIR/NAME.SIG, with he parameter file called PARAM.
# NB: PARAM must be set to UNDEFINED_PARAMETER when the progam is compiled without parameter or it is not parametrised.
# A test is a set of calls to the Signal compiler for testing every options of the batch compiler.
# Such a test is defined by a line (3 values) of the global variable LIST_TEST_SYSTEMATIC_OPTION.
# It calls GEN_ONE_FILE_OPTION()
FUNCTION(PK_GEN_TEST NAMETEST SRCDIR NAME PARAM)
  SET(OPTIONS_TEST "tra")
  SET(OPTIONSUFFIX  ${UNDEFINED_PARAMETER})
  SET(TRGTDIRPREFIX ${SRCDIR}/${NAME}/Test_${NAMETEST})
  
  GET_OPTIONS_TO_TEST(OPTIONS_TO_BE_TESTED)
#  MESSAGE("GET_OPTIONS_TO_TEST= ${OPTIONS_TO_BE_TESTED}")

  LIST(LENGTH OPTIONS_TO_BE_TESTED L)
  MATH(EXPR L " ${L} - 1")
  FOREACH(I RANGE 0 ${L} 3)
    LIST(GET OPTIONS_TO_BE_TESTED ${I} OPTIONS_TEST)
    MATH(EXPR X "${I} + 1")
    LIST(GET OPTIONS_TO_BE_TESTED ${X} OPTIONSUFFIX)
    MATH(EXPR X "${I} + 2")
    LIST(GET OPTIONS_TO_BE_TESTED ${X} CHECKFILES)
    SEPARATE_ARGUMENTS(CHECKFILES)
    GEN_ONE_FILE_OPTION(${NAME} ${PARAM} ${SRCDIR} ${OPTIONS_TEST} ${TRGTDIRPREFIX} ${OPTIONSUFFIX} "${CHECKFILES}")
  ENDFOREACH(I RANGE 0 ${L} 4)

  SET(OPTIONS_TO_BE_TESTED)
ENDFUNCTION()

# It adds a test for the SIGNAL program SRCDIR/NAME.SIG with the parameter file NAME.PAR if it exists in SRCDIR.
# It calls the  PK_GEN_TEST()
FUNCTION(PK_ADD_TEST_DIR SRCDIR NAME)
  SET(NAMETEST "SYSTEMATIC_OPTION")
  SET(PARAM ${NAME}.PAR)
  IF(NOT EXISTS "${SRCDIR}/${PARAM}")
    SET(PARAM ${UNDEFINED_PARAMETER})
  ENDIF(NOT EXISTS "${SRCDIR}/${PARAM}")
  PK_GEN_TEST(${NAMETEST} ${SRCDIR} ${NAME} ${PARAM})
ENDFUNCTION(PK_ADD_TEST_DIR SRCDIR NAME)

# It adds a test for all the signal programs (files suffied by .SIG) of the SRCDIR direcory.
# Send an error when the directory does not exist.
# It calls PK_ADD_TEST_DIR() for each selected file.
FUNCTION(PK_ADD_TEST_ALL_DIR SRCDIR)
  IF (NOT EXISTS ${SRCDIR})
    PK_ERROR("PK_ADD_TEST_ALL_DIR(1), ${MESS1} ${SRCDIR}")
    RETURN()
  ENDIF()

  FILE(GLOB TESTFILES "${SRCDIR}/*.SIG")
  PRINT("TESTFILES = ${TESTFILES}")
  FOREACH(TESTFILE ${TESTFILES})
    STRING(REPLACE "${SRCDIR}" " " NAME ${TESTFILE})
    STRING(REPLACE ".SIG" " " NAME ${NAME})
    STRING(REPLACE "/" " " NAME ${NAME})
    STRING(REPLACE "\\" " " NAME ${NAME})
    STRING(STRIP ${NAME} NAME)
    PK_ADD_TEST_DIR(${SRCDIR} ${NAME})
  ENDFOREACH(TESTFILE ${TESTFILES})
ENDFUNCTION()


# --- The followings are currently the Public ones.
# VarSrcDir is a name of a cmake variable (example: ex_assertions) 
FUNCTION(PK_ADD_TEST_SUBDIR VarSrcDir LSUBDIR)
  GET_DIR_NAME(${VarSrcDir} DIR)
  PK_ADD_TEST_SUBDIR_PATH(${DIR} ${LSUBDIR})
ENDFUNCTION()

FUNCTION(PK_ADD_TEST_ALL VarSrcDir)
  GET_DIR_NAME(${VarSrcDir} SRCDIR)
  PK_ADD_TEST_ALL_PATH(${SRCDIR})
ENDFUNCTION(PK_ADD_TEST_ALL SRCDIR )


FUNCTION(PK_ADD_TEST VarSrcDir NAME)
  GET_DIR_NAME(${VarSrcDir} SRCDIR)
  PK_ADD_TEST_PATH(${SRCDIR} ${NAME})
ENDFUNCTION()

FUNCTION(PK_ADD_TEST_PAR VarSrcDir NAME PARAM)
  GET_DIR_NAME(${VarSrcDir} SRCDIR)
  PK_ADD_TEST_PAR_PATH(${SRCDIR} ${NAME} ${PARAM})
ENDFUNCTION()


# The following works on the paths

FUNCTION(PK_ADD_TEST_PAR_PATH SRCDIR NAME PARAM)
  IF(DEFINED SRCDIR)
    SET(NAMETEST "SYSTEMATIC_OPTION_PARAM")
    PK_GEN_TEST(${NAMETEST} ${SRCDIR} ${NAME} ${PARAM})
  ENDIF()
ENDFUNCTION()


FUNCTION(PK_ADD_TEST_PATH SRCDIR NAME)
  IF(DEFINED SRCDIR)
    PK_ADD_TEST_DIR(${SRCDIR} ${NAME})
  ENDIF()
ENDFUNCTION()


FUNCTION(PK_ADD_TEST_ALL_PATH SRCDIR)
  IF(DEFINED SRCDIR)
    PK_ADD_TEST_ALL_DIR(${SRCDIR})
  ENDIF()
ENDFUNCTION()

FUNCTION(PK_ADD_TEST_SUBDIR_PATH DIR LSUBDIR)
  IF(DEFINED DIR)
    FOREACH(CUR_DIR ${LSUBDIR})
      IF(IS_DIRECTORY  ${DIR}/${CUR_DIR})
	IF(NOT ${CUR_DIR} STREQUAL "cmake")
	  # MESSAGE(" PK_ADD_TEST_ALL_DIR ${DIR}/${CUR_DIR}")
	  PK_ADD_TEST_ALL_DIR("${DIR}/${CUR_DIR}")
	ENDIF()
      ENDIF()
    ENDFOREACH()
  ENDIF()
ENDFUNCTION()

