# CMAKE utility, used to fix the architecture type (32/64 bits) search CHECK_TYPE_SIZE
INCLUDE(CheckTypeSize) 

# --------------------------------------------------------------------------------------------------
# It defines in the cmake variable ARCHI by 32 or 64 bits.
# --------------------------------------------------------------------------------------------------
MACRO(SET_ARCHI)
  SET(ARCHI 32)
  IF(UNIX)
    CHECK_TYPE_SIZE(long TS)
    STRING(COMPARE EQUAL  ${TS} "8" RES)
    IF(RES)
      SET(ARCHI 64)
    ENDIF(RES)
  ELSE(UNIX)
    IF (CMAKE_CL_64)
      SET(ARCHI 64)
    ENDIF(CMAKE_CL_64)
  ENDIF(UNIX)	
  PRINT ("ARCHI= ${ARCHI} BITS")
ENDMACRO(SET_ARCHI)

# --------------------------------------------------------------------------------
# It defines 
#   - options for the C/C++ compilers 
#   - some C/C++ symbols
#   - ARCHI cmake variable that defines the type of architecutre (value 32 or  64)
# --------------------------------------------------------------------------------
MACRO(SET_C_CPP_DEFS)
  SET_ARCHI() # Defining the symbol ARCHI=32 | 64
  SET(CPP_FLAGS)
  IF( UNIX )
    # FLAGS COMPILING UNIX
    SET(CPP_FLAGS " -g -fPIC ")  # Debug, add -w (for without warnings)
    SET(CPP_FLAGS " -fPIC ")   # Optimisations. (don't use -O3)
    ADD_DEFINITIONS( -D_HAVE_${ARCHI}_BITS)
    # ADD_DEFINITIONS( -pedantic -Wall ) # -std=c++98 (only for C++), -ansi (but // not allowed in C)
  ELSE(UNIX)
    IF (MSVC)
#      SET(CPP_FLAGS "/nologo /W3 /c /MDd /Gm /EHsc /Zi ") # the option /RTC1 must not be used
#      SET(CPP_FLAGS "/nologo /W3 /c /EHsc ") # the option /RTC1 must not be used
       ADD_DEFINITIONS( /DWIN${ARCHI} /D_HAVE_${ARCHI}_BITS )
       ADD_DEFINITIONS( /D_MBCFS )
       ADD_DEFINITIONS( /D_LIB )
       ADD_DEFINITIONS( /D__STDC__ /D_CRT_SECURE_NO_WARNINGS )
       ADD_DEFINITIONS( /D__PKWINDOWS__ )  # added for distinguish MSVC and MinGW
      # ADD_DEFINITIONS( /DLIBXML_STATIC /DLIBXSLT_STATIC /DLIBEXSLT_STATIC /DXMLSEC_STATIC )  # added,Sept 2015 LBesnard, for XML2 (required for windows 64) 
     ELSE (MSVC) # MinGW
       SET(CPP_FLAGS " -g")
       ADD_DEFINITIONS( -D_HAVE_${ARCHI}_BITS )
     ENDIF( MSVC )
  ENDIF( UNIX )
ENDMACRO(SET_C_CPP_DEFS)

# ----------------------------------------------------------------------
# Definition of the internal symbols used in the SignalToolbox C/C++ code
# ----------------------------------------------------------------------.
MACRO(SET_CPP_SYMBOLS)
  ADD_DEFINITIONS( -D_VIRTUAL_=)
  ADD_DEFINITIONS( -D_PROTECTED_= )
  ADD_DEFINITIONS( -D_PUBLIC_= )
  ADD_DEFINITIONS( -D_PRIVATE_= )
  ADD_DEFINITIONS( -D_STATIC_=)
#  ADD_DEFINITIONS( -D_INIT_=: )
#  ADD_DEFINITIONS( -D_ENDINIT_= )
ENDMACRO(SET_CPP_SYMBOLS)

# --------------------------------------------------------------------------------------------------------
# It adds the definition (define symbol in C/C++) of the xKHOST symbol that specifies the operation system.
# xKHOST= -1 for unkown, 5 for unix, 10 for windows, and 15 for Mac, 
# This symbol is used in the source of the software: the C/C++ compiler is called with these definitions.
# --------------------------------------------------------------------------------------------------------
MACRO(SET_HOST_DEFS)
  if( APPLE )
    ADD_DEFINITIONS( -DxKHOST=15 )
  ELSE( APPLE )
    if( UNIX )
      SET(STATIC_LINK_OPTION "-static")
      ADD_DEFINITIONS( -DxKHOST=5 )
    ELSE( UNIX )
      IF (MSVC OR MINGW)
	ADD_DEFINITIONS( -DxKHOST=10)  #window
      ELSE(MSVC OR MINGW)
	ADD_DEFINITIONS( -DxKHOST=-1)  #UNKNOWN
      ENDIF( MSVC OR MINGW )
    endif( UNIX )
  ENDIF( APPLE )
ENDMACRO(SET_HOST_DEFS)

# ---------------------------------------------------------------------------
# It adds  the definition (define symbol in C/C++) of the LITTLEENDIAN symbol 
# that specifies the the system is big endian.
# ---------------------------------------------------------------------------
MACRO(SETLITTLEENDIAN)
  INCLUDE(TestBigEndian)
  TEST_BIG_ENDIAN(XBIGENDIAN)
  PRINT("Test BIGENDIAN returns = ${XBIGENDIAN}")
  ADD_DEFINITIONS( -DLITTLEENDIAN=${XBIGENDIAN})
ENDMACRO(SETLITTLEENDIAN)
