#include "QueuingPortManager.h"


/*****************************************   QUEUING PORT OBJECT ****************************************/


QueuingPort::QueuingPort (APEX_QueuingPort_type s):queuingport (s)
{
}




/************************************** QUEUING PORT MANAGER_NODE  **************************************/

QueuingPortManagerNode::QueuingPortManagerNode (QueuingPortAssoc_type contents):next (NULL)
{
  Contents.QueuingPort_ID = contents.QueuingPort_ID;
  Contents.ptrQueuingPort = contents.ptrQueuingPort;
}

QueuingPortManagerNode::QueuingPortManagerNode ()
{
}

QueuingPortManagerNode::~QueuingPortManagerNode ()
{
}

void
QueuingPortManagerNode::setContents (QueuingPortAssoc_type contents)
{
  Contents.QueuingPort_ID = contents.QueuingPort_ID;
  Contents.ptrQueuingPort = contents.ptrQueuingPort;
}

void
QueuingPortManagerNode::setNext (ptQueuingPortManagerNode node)
{
  next = node;
}

QueuingPortAssoc_type
QueuingPortManagerNode::getContents ()
{
  return Contents;
}

ptQueuingPortManagerNode
QueuingPortManagerNode::getNext ()
{
  return next;
}



/*************************************** QUEUING PORT MANAGER  *****************************************/

QueuingPortManager::QueuingPortManager (int size):first (NULL), last (NULL), max_size (size), current_size (0)
{
}

QueuingPortManager::QueuingPortManager ():first (NULL), last (NULL), max_size (MAX_NUMBER_OF_QUEUING_PORTS), current_size (0)
{
}

QueuingPortManager::~QueuingPortManager ()
{
}

bool
QueuingPortManager::getElemNamed (Comm_ComponentName_type queuingport_Name, ptQueuingPortManagerNode * elem)
{
  ptQueuingPortManagerNode iter = first;

  while (iter != NULL)
    if (strcmp ((((iter->getContents ()).ptrQueuingPort)->queuingport).QueuingPort_Name, queuingport_Name) == 0)
      {
	*elem = iter;
	return pk_true;
      }
    else
      iter = iter->getNext ();
  return pk_false;
}

bool
QueuingPortManager::getElemId (Comm_ComponentID_type queuingport_ID, ptQueuingPortManagerNode * elem)
{
  ptQueuingPortManagerNode iter = first;

  while (iter != NULL)
    if ((iter->getContents ()).QueuingPort_ID == queuingport_ID)
      {
	*elem = iter;
	return pk_true;
      }
    else
      iter = iter->getNext ();
  return pk_false;
}

void
QueuingPortManager::setFirst (ptQueuingPortManagerNode node)
{
  first = node;
}

void
QueuingPortManager::setLast (ptQueuingPortManagerNode node)
{
  last = node;
}

ptQueuingPortManagerNode
QueuingPortManager::getFirst ()
{
  return first;
}


ptQueuingPortManagerNode
QueuingPortManager::getLast ()
{
  return last;
}


bool
QueuingPortManager::QueuingPortRecord (Comm_ComponentName_type queuingport_Name,
				       QueuingPortSize_type queuingPort_Size,
				       PortDirection_type port_direction,
				       MessageSize_type message_size,
				       QueuingDiscipline_type queuing_discipline, Comm_ComponentID_type & queuingport_ID)
{
  if (current_size < max_size)
    {
      current_size++;
      QueuingPortAssoc_type new_record;
      new_record.QueuingPort_ID = current_size;

      APEX_QueuingPort_type queuingport;

      strcpy (queuingport.QueuingPort_Name, queuingport_Name);
      queuingport.QueuingPort_Size = queuingPort_Size;
      queuingport.Port_Direction = port_direction;
      queuingport.Message_Size = message_size;
      queuingport.Nb_Message = 0;
      queuingport.Queuing_Discipline = queuing_discipline;
      queuingport.WaitingOnSend_Processes = 0;
      queuingport.WaitingOnReceive_Processes = 0;

      new_record.ptrQueuingPort = new QueuingPort (queuingport);
      queuingport_ID = current_size;

      ptQueuingPortManagerNode new_node = new QueuingPortManagerNode (new_record);

      if (first == NULL)
	{
	  first = new_node;
	  last = new_node;
	}
      else
	{
	  ptQueuingPortManagerNode iter = first;
	  while (iter != last)
	    iter = iter->getNext ();
	  last->setNext (new_node);
	  last = new_node;
	};
      return pk_true;
    }
  return pk_false;
}


bool
QueuingPortManager::QueuingPortCheckName (Comm_ComponentName_type queuingport_Name, Comm_ComponentID_type & queuingport_ID)
{
  bool retour;
  ptQueuingPortManagerNode qport;

  if (!getElemNamed (queuingport_Name, &qport))
    {
      queuingport_ID = NULL;
      retour = pk_false;
    }
  else
    {
      queuingport_ID = (qport->getContents ()).QueuingPort_ID;
      retour = pk_true;
    }
  return retour;
}


bool
QueuingPortManager::QueuingPortCheckID (Comm_ComponentID_type queuingport_ID, APEX_QueuingPort_type & queuingport)
{
  ptQueuingPortManagerNode qport;

  if (!getElemId (queuingport_ID, &qport))
    return pk_false;
  queuingport = ((qport->getContents ()).ptrQueuingPort)->queuingport;
  return pk_true;
}



void
QueuingPortManager::QueuingPortUpdate (APEX_QueuingPort_type queuingport)
{
  APEX_QueuingPort_type *x;
  ptQueuingPortManagerNode qport;

  // WHY Comm_ComponentName_type name; strcpy(name, queuingport.QueuingPort_Name);
  if (getElemNamed (queuingport.QueuingPort_Name, &qport))
    {
      x = &(((qport->getContents ()).ptrQueuingPort)->queuingport);
      x->QueuingPort_Size = queuingport.QueuingPort_Size;
      x->Port_Direction = queuingport.Port_Direction;
      x->Message_Size = queuingport.Message_Size;
      x->Nb_Message = queuingport.Nb_Message;
      for (int i = 0; i < MAX_QUEUING_PORT_SIZE; i++)
	x->Message_Range[i] = queuingport.Message_Range[i];
      x->Queuing_Discipline = queuingport.Queuing_Discipline;
      x->WaitingOnSend_Processes = queuingport.WaitingOnSend_Processes;
      x->WaitingOnReceive_Processes = queuingport.WaitingOnReceive_Processes;
      x->WaitingOnSend_Queue = queuingport.WaitingOnSend_Queue;
      x->WaitingOnReceive_Queue = queuingport.WaitingOnReceive_Queue;
    }
}

bool
QueuingPortManager::Full ()
{
  return (max_size == current_size);
}


/* void QueuingPortManager::printFile(Comm_ComponentID_type queuingport_ID, char *result_file)
   { 
   FILE *file_out;
   APEX_QueuingPort_type queuingport;
   if (this->QueuingPortCheckID(queuingport_ID, queuingport))
   { file_out = fopen(result_file,"wt");
   if (!file_out)
   {
   fprintf(stderr,"Can not open file %s\n",result_file);
   exit(1);
   };    
   fprintf(file_out, "(QueuingPort_Identifier : %i, Name : %s, Displayed_Message : %i, Empty_Indicator : %i, Waiting_Processes : %i) \n" ,queuingport_ID, queuingport.QueuingPort_Name, queuingport.Message, queuingport.Empty_Indicator, queuingport.Waiting_Processes);
   fclose(file_out);
   }
   }
*/
