#include "TYPES.h"
#include "FIFO_ReceiveQueue.h"
#include "PRIORITY_ReceiveQueue.h"

#include "SemaphoreManager.h"




/**********************************************************************************/
/****************************   SEMAPHORE_OBJECT : BEGIN **************************/
/**********************************************************************************/


Semaphore::Semaphore (APEX_Semaphore_type s):sema (s)
{
}


/**********************************************************************************/
/**********************************  MANAGER_NODE  ********************************/
/**********************************************************************************/


SemaphoreManagerNode::SemaphoreManagerNode (SemAssoc_type value):next (NULL)
{
  contents.SemaphoreID = value.SemaphoreID;
  contents.ptrSemaphore = value.ptrSemaphore;
}

SemaphoreManagerNode::SemaphoreManagerNode ()
{
}

SemaphoreManagerNode::~SemaphoreManagerNode ()
{
}

void
SemaphoreManagerNode::setValue (SemAssoc_type value)
{
  contents.SemaphoreID = value.SemaphoreID;
  contents.ptrSemaphore = value.ptrSemaphore;
}

void
SemaphoreManagerNode::setNext (ptSemaphoreManagerNode ptvalue)
{
  next = ptvalue;
}

SemAssoc_type
SemaphoreManagerNode::getValue ()
{
  return contents;
}

ptSemaphoreManagerNode
SemaphoreManagerNode::getNext ()
{
  return next;
}


/**********************************************************************************/
/*************************************  MANAGER  **********************************/
/**********************************************************************************/


SemaphoreManager::SemaphoreManager (int size):first (NULL), last (NULL), max_size (size), current_size (0)
{
}

SemaphoreManager::SemaphoreManager ():first (NULL), last (NULL), max_size (MAX_NUMBER_OF_SEMAPHORES), current_size (0)
{
}

SemaphoreManager::~SemaphoreManager ()
{
}

void
SemaphoreManager::setFirst (ptSemaphoreManagerNode ptvalue)
{
  first = ptvalue;
}

void
SemaphoreManager::setLast (ptSemaphoreManagerNode ptvalue)
{
  last = ptvalue;
}

ptSemaphoreManagerNode
SemaphoreManager::getFirst ()
{
  return first;
}

ptSemaphoreManagerNode
SemaphoreManager::getLast ()
{
  return last;
}

bool
SemaphoreManager::SemaphoreRecord (Comm_ComponentName_type semaphore_Name,
				   int current_Value,
				   int maximum_Value, QueuingDiscipline_type queuing_Discipline, Comm_ComponentID_type & semaphoreID)
{
  if (current_size < max_size)
    {
      current_size++;
      SemAssoc_type new_record;
      new_record.SemaphoreID = current_size;

      APEX_Semaphore_type sema;
      strcpy (sema.Semaphore_Name, semaphore_Name);
      sema.Current_Value = current_Value;
      sema.Maximum_Value = maximum_Value;
      sema.Queuing_Discipline = queuing_Discipline;
      sema.Waiting_Processes = 0;
      if (queuing_Discipline == FIFO)
	{
	  sema.Waiting_Queue = new FIFO_ReceiveQueue ();
	}
      else
	{
	  sema.Waiting_Queue = new PRIORITY_ReceiveQueue ();
	};

      ptSemaphore ptsem = new Semaphore (sema);
      new_record.ptrSemaphore = new Semaphore (sema);
      semaphoreID = current_size;

      ptSemaphoreManagerNode new_node = new SemaphoreManagerNode (new_record);
      if (first == NULL)
	{
	  first = new_node;
	  last = new_node;
	}
      else
	{
	  ptSemaphoreManagerNode iter = first;
	  while (iter != last)
	    {
	      iter = iter->getNext ();
	    };
	  if (iter == last)
	    {
	      last->setNext (new_node);
	      last = new_node;
	    }

	};
      return pk_true;
    }
  else
    {
      return pk_false;
    };
}

bool
SemaphoreManager::SemaphoreCheckName (Comm_ComponentName_type semaphoreName, Comm_ComponentID_type & semaphoreID)
{
  bool retour;
  if (first == NULL)
    {
      semaphoreID = NULL;
      retour = pk_false;
    }
  else
    {
      ptSemaphoreManagerNode iter = first;
      while (strcmp ((((iter->getValue ()).ptrSemaphore)->sema).Semaphore_Name, semaphoreName) && (iter != last))
	{
	  iter = iter->getNext ();
	};
      if (iter == last)
	{
	  if (strcmp ((((iter->getValue ()).ptrSemaphore)->sema).Semaphore_Name, semaphoreName))
	    {
	      semaphoreID = NULL;
	      retour = pk_false;
	    }
	  else
	    {
	      semaphoreID = (iter->getValue ()).SemaphoreID;
	      retour = pk_true;
	    }
	}
      else
	{
	  semaphoreID = (iter->getValue ()).SemaphoreID;
	  retour = pk_true;
	}
    };
  return retour;
}

bool
SemaphoreManager::SemaphoreCheckID (Comm_ComponentID_type semaphoreID, APEX_Semaphore_type & sema)
{
  bool retour;
  if (first == NULL)
    {
      retour = pk_false;
    }
  else
    {
      ptSemaphoreManagerNode iter = first;
      while (((iter->getValue ()).SemaphoreID != semaphoreID) && (iter != last))
	{
	  iter = iter->getNext ();
	};
      if (iter == last)
	{
	  if ((iter->getValue ()).SemaphoreID != semaphoreID)
	    {
	      retour = pk_false;
	    }
	  else
	    {
	      sema = ((last->getValue ()).ptrSemaphore)->sema;
	      retour = pk_true;
	    }
	}
      else
	{
	  sema = ((iter->getValue ()).ptrSemaphore)->sema;
	  retour = pk_true;
	}
    };
  return retour;
}


void
SemaphoreManager::SemaphoreUpdate (APEX_Semaphore_type sema)
{
  if (first != NULL)
    {
      ptSemaphoreManagerNode iter = first;
      Comm_ComponentName_type name;
      strcpy (name, sema.Semaphore_Name);
      while (strcmp ((((iter->getValue ()).ptrSemaphore)->sema).Semaphore_Name, name) && (iter != last))
	{
	  iter = iter->getNext ();
	};
      if (iter == last)
	{
	  if (!strcmp ((((iter->getValue ()).ptrSemaphore)->sema).Semaphore_Name, name))
	    {
	      (((iter->getValue ()).ptrSemaphore)->sema).Current_Value = sema.Current_Value;
	      (((iter->getValue ()).ptrSemaphore)->sema).Maximum_Value = sema.Maximum_Value;
	      (((iter->getValue ()).ptrSemaphore)->sema).Waiting_Processes = sema.Waiting_Processes;
	      (((iter->getValue ()).ptrSemaphore)->sema).Queuing_Discipline = sema.Queuing_Discipline;
	      (((iter->getValue ()).ptrSemaphore)->sema).Waiting_Queue = sema.Waiting_Queue;
	    }
	}
      else
	{
	  (((iter->getValue ()).ptrSemaphore)->sema).Current_Value = sema.Current_Value;
	  (((iter->getValue ()).ptrSemaphore)->sema).Maximum_Value = sema.Maximum_Value;
	  (((iter->getValue ()).ptrSemaphore)->sema).Waiting_Processes = sema.Waiting_Processes;
	  (((iter->getValue ()).ptrSemaphore)->sema).Queuing_Discipline = sema.Queuing_Discipline;
	  (((iter->getValue ()).ptrSemaphore)->sema).Waiting_Queue = sema.Waiting_Queue;
	};
    };
}



bool
SemaphoreManager::Full ()
{
  return (max_size == current_size);
}


void
SemaphoreManager::printFile (Comm_ComponentID_type semaphoreID, char *result_file)
{
  FILE *file_out;
  APEX_Semaphore_type sema;
  if (this->SemaphoreCheckID (semaphoreID, sema))
    {
      file_out = fopen (result_file, "wt");
      if (!file_out)
	{
	  fprintf (stderr, "Can not open file %s\n", result_file);
	  exit (1);
	};
      fprintf (file_out,
	       "(Semaphore_Identifier : %i, Name : %s, Semaphore_Current_Val : %i, Semaphore_Maximum_Val : %i,Waiting_Processes : %i, Queuing_Discipline : %i) \n",
	       semaphoreID, sema.Semaphore_Name, sema.Current_Value, sema.Maximum_Value, sema.Waiting_Processes, sema.Queuing_Discipline);
      fclose (file_out);
    }
}
