#include "TYPES.h"
#include "FIFO_SendQueue.h"




/************************************************************************************************/
/************************************   FIFO_WAITING_QUEUE_NODE   *******************************/
/************************************************************************************************/


FIFO_SendQueueNode::FIFO_SendQueueNode ()
{
}

FIFO_SendQueueNode::FIFO_SendQueueNode (const ProcMess_type & PM):
next (NULL)
{
  contents.Proc = PM.Proc;
  contents.Mess = PM.Mess;
}

void
FIFO_SendQueueNode::setContents (ProcMess_type PM)
{
  contents.Proc = PM.Proc;
  contents.Mess = PM.Mess;
}

void
FIFO_SendQueueNode::setNext (ptFIFO_SendQueueNode nextPM)
{
  next = nextPM;
}

ProcMess_type
FIFO_SendQueueNode::getContents ()
{
  return contents;
}

ptFIFO_SendQueueNode
FIFO_SendQueueNode::getNext ()
{
  return next;
}





/**********************************************************************************************/
/**************************************   FIFO_SENDQUEUE  *************************************/
/**********************************************************************************************/


FIFO_SendQueue::FIFO_SendQueue ():first (NULL), last (NULL), max_size (MAX_NUMBER_OF_PROCESSES), current_size (0)
{
}

FIFO_SendQueue::FIFO_SendQueue (int size):first (NULL), last (NULL), max_size (size), current_size (0)
{
}

void
FIFO_SendQueue::insert (ProcessID_type Process_ID, Message_type Message)
{
  if (current_size < max_size)
    {
      ProcMess_type PM;
      PM.Proc = Process_ID;
      PM.Mess = Message;
      ptFIFO_SendQueueNode node;

      node = new FIFO_SendQueueNode (PM);
      if (first == NULL)
	{
	  first = node;
	  last = node;
	}
      else
	{
	  last->setNext (node);
	  last = node;
	}
      current_size++;
    };
}


void
FIFO_SendQueue::setFirst (ptFIFO_SendQueueNode Node)
{
  first = Node;
}


void
FIFO_SendQueue::setLast (ptFIFO_SendQueueNode Node)
{
  last = Node;
}


ptFIFO_SendQueueNode
FIFO_SendQueue::getFirst ()
{
  return first;
}


ptFIFO_SendQueueNode
FIFO_SendQueue::getLast ()
{
  return last;
}


int
FIFO_SendQueue::getMax_size ()
{
  return max_size;
}


int
FIFO_SendQueue::getCurrent_size ()
{
  return current_size;
}


ProcessID_type
FIFO_SendQueue::remove (Message_type & Message)
{
  ProcMess_type PM;
  if (!(this->empty ()))
    {
      ptFIFO_SendQueueNode node = new FIFO_SendQueueNode ();
      node = first;
      first = node->getNext ();
      if (last == node)
	last = NULL;
      PM = node->getContents ();
      current_size--;
    };
  Message = PM.Mess;
  return PM.Proc;
}

bool
FIFO_SendQueue::empty ()
{
  return (first == NULL);
}

bool
FIFO_SendQueue::full ()
{
  return (max_size == current_size);
}


void
FIFO_SendQueue::printFile (char *result_file)
{
  FILE *file_out;
  ptFIFO_SendQueueNode iter = first;

  file_out = fopen (result_file, "wt");
  if (!file_out)
    {
      fprintf (stderr, "Can not open file %s\n", result_file);
      exit (1);
    };
  while (iter != NULL)
    {
      ProcMess_type val = iter->getContents ();
      fprintf (file_out, "(%d, %d) \n", val.Proc, val.Mess);
      iter = iter->getNext ();
    };
  fclose (file_out);
}
